#include "TFFTRealComplex.h"
#include "TFFTComplexReal.h"
#include "TH1D.h"
#include "TROOT.h"
#include <iostream>
#include "TH2D.h"
#include "utils.h"
#include "doFFT_ROOT.h"
#include "bp_filter.h"//Filter
     
using namespace std;


FFTdata_ROOT::FFTdata_ROOT():FFTdata(){
  hKspaceReal=new TH1D;//("hKspaceReal","hKspaceReal",1,0,1);
  hKspaceImaginary=new TH1D;//("hKspaceImaginary","hKspaceImaginary",1,0,1);
  hRspaceReal=new TH1D;//("hRspaceReal","hRspaceReal",1,0,1);
}

FFTdata_ROOT::~FFTdata_ROOT(){
  delete hKspaceReal;
  delete hKspaceImaginary;
  delete hRspaceReal;

}

void FFTdata_ROOT::Transform(const TH1D *h1){
  //copy to in data
  hRspaceReal->SetBins(h1->GetNbinsX(),
		       h1->GetXaxis()->GetXmin(),
		       h1->GetXaxis()->GetXmax());
  for (int i=0;i<h1->GetNbinsX();i++) (*hRspaceReal)[i+1]=(*h1)[i+1];
  
  TFFTRealComplex fft(h1->GetNbinsX(),kFALSE);
  

  //init transformation
  fft.Init("M",0,0);
  //set data
  for (int i=0;i<h1->GetNbinsX();i++) fft.SetPoint(i,h1->GetBinContent(i+1));
  fft.Transform();
  
  //there are n/2+1 complex numbers in the output;
  double f0=1/(h1->GetXaxis()->GetXmax()-h1->GetXaxis()->GetXmin());
  int n2=h1->GetNbinsX()/2+1;

  //TH1D *hReal=getHistogram<TH1D>("hReal");
  
  hKspaceReal->SetBins(n2,-0.5*f0,(n2-0.5)*f0);
  hKspaceReal->Reset();

  //TH1D *hIm=getHistogram<TH1D>("hImaginary");
  hKspaceImaginary->SetBins(n2,-0.5*f0,(n2-0.5)*f0);
  hKspaceImaginary->Reset();

  //TH1D *hAmp=getHistogram<TH1D>("hAmplitude");
  //hAmp->SetBins(n2,-0.5*f0,(n2-0.5)*f0);
  //hAmp->Reset();

  //store transformation
  for (int i=0;i<n2;i++) {
    double re,im;
    fft.GetPointComplex(i,re,im,kFALSE);
    hKspaceReal->SetBinContent(i+1,re);
    hKspaceImaginary->SetBinContent(i+1,im);
    //hAmp->SetBinContent(i+1,sqrt(re*re+im*im));
  }

}

void FFTdata_ROOT::InverseTransform(TH1D *h1){
  //look to gROOT for transformations and restore them into h1
  // TH1D *hRe=dynamic_cast<TH1D *>(gROOT->FindObject("hReal"));
  //   if (!hRe){
  //     cout<<"Could not find input histogram hReal!"<<endl;
  //     return;
  //   }
  //   TH1D *hIm=dynamic_cast<TH1D *>(gROOT->FindObject("hImaginary"));
  //   if (!hIm){
  //     cout<<"Could not find input histogram hImaginary!"<<endl;
  //     return;
  //   }
  
  int n=2*hKspaceReal->GetNbinsX()-1;
  TFFTComplexReal fft(n,kFALSE);
  fft.Init("M",0,0);
  
  for (int i=0;i<hKspaceReal->GetNbinsX();i++){
    double re=hKspaceReal->GetBinContent(i+1);
    double im=hKspaceImaginary->GetBinContent(i+1);
    fft.SetPoint(i,re,im);
  }
  
  fft.Transform();
  double f0=-2*hKspaceReal->GetXaxis()->GetXmin();

  hRspaceReal->SetBins(n,-0.5/f0,0.5/f0);
  for (int i=0;i<hRspaceReal->GetNbinsX();i++) 
    hRspaceReal->SetBinContent(i+1,fft.GetPointReal(i,kFALSE));
  
  hRspaceReal->Scale(1./h1->GetNbinsX());
  
  hRspaceReal->TArrayD::Copy(*h1);
  

}

void FFTdata_ROOT::ApplyFilter(Filter *filter){
  
  if (!filter){
    cout<<"Supply a valid filter!"<<endl;
    return;
  }

   //look to gROOT for transformations and restore them into h1
  // TH1D *hRe=dynamic_cast<TH1D *>(gROOT->FindObject("hReal"));
//   if (!hRe){
//     cout<<"Could not find input histogram hReal!"<<endl;
//     return;
//   }
  
//   TH1D *hIm=dynamic_cast<TH1D *>(gROOT->FindObject("hImaginary"));
//   if (!hIm){
//     cout<<"Could not find input histogram hImaginary!"<<endl;
//     return;
//   }
  
  for (int i=0;i<hKspaceReal->GetNbinsX();i++){
    double freq=hKspaceReal->GetXaxis()->GetBinCenter(i+1);
    freq/=hKspaceReal->GetXaxis()->GetBinCenter(hKspaceReal->GetNbinsX());
    (*hKspaceReal)[i+1]*=(*filter)(freq);
    (*hKspaceImaginary)[i+1]*=(*filter)(freq);
  }
}


void FFTdata_ROOT::Divide (const FFTdata & fft_right){
  const FFTdata_ROOT *ft=dynamic_cast<const FFTdata_ROOT *>(&fft_right);
  if (!ft) return;
  
  if (hKspaceReal->GetNbinsX()!=ft->GetKspaceReal()->GetNbinsX()) return;
  
  
  for (int i=0;i<hKspaceReal->GetNbinsX();i++){
    //calculate (a+bi)/(c+di) in complex
    double a=(*hKspaceReal)[i+1];
    double b=(*hKspaceImaginary)[i+1];

    //divided by
    double c=(*ft->GetKspaceReal())[i+1];
    double d=(*ft->GetKspaceImaginary())[i+1];
    
    double A=c*c+d*d;
    double fR=a*c+b*d;
    double fI=b*c-a*d;
    
    if (A==0 && (fR!=0 || fI!=0)){
      cout<<"Zero division for frequency "<<i<<endl;
      return;
    }
    if (A==0){
      (*hKspaceReal)[i]=0;
      (*hKspaceImaginary)[i]=0;
      continue;
    }
    (*hKspaceReal)[i]=fR/A;
    (*hKspaceImaginary)[i]=fI/A;
  }
  
  
}


void FFTdata_ROOT::Multiply(const FFTdata& fft_right){
  const FFTdata_ROOT *ft=dynamic_cast<const FFTdata_ROOT *>(&fft_right);
  if (!ft) return;
  
  if (hKspaceReal->GetNbinsX()!=ft->GetKspaceReal()->GetNbinsX()) return;
  
  
  for (int i=0;i<hKspaceReal->GetNbinsX();i++){
    //calculate (a+bi)*(c+di) in complex
    double a=(*hKspaceReal)[i+1];
    double b=(*hKspaceImaginary)[i+1];

    //divided by
    double c=(*ft->GetKspaceReal())[i+1];
    double d=(*ft->GetKspaceImaginary())[i+1];
    
    double fR=a*c-b*d;
    double fI=b*c+a*d;
    
    (*hKspaceReal)[i+1]=fR;
    (*hKspaceImaginary)[i+1]=fI;
  }
  
}

TH1D *FFTdata_ROOT::GetPowerSpectrum(TH1D *placeHolder){
  placeHolder->SetBins(hKspaceReal->GetNbinsX(),
		       hKspaceReal->GetXaxis()->GetXmin(),
		       hKspaceReal->GetXaxis()->GetXmax());

  for (int i=0;i<placeHolder->GetNbinsX();i++){
    double re=hKspaceReal->GetBinContent(i+1);
    double im=hKspaceImaginary->GetBinContent(i+1);
    placeHolder->SetBinContent(i+1,re*re+im*im);
    
  }
  return placeHolder;
}

