#include "utils.h"
#include <cmath>
#include <sstream>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include "TRandom3.h"
#include <limits>
#include <cstdlib>
#include <libgen.h>//dirname
#include <fstream>
#include "TMath.h"

using namespace std;

TH2D *SetBins(TH2D *data, const TH2D *pattern){
  //set data to have the same bins as pattern
  data->SetBins(
		pattern->GetNbinsX(),
		pattern->GetXaxis()->GetXmin(),
		pattern->GetXaxis()->GetXmax(),
		pattern->GetNbinsY(),
		pattern->GetYaxis()->GetXmin(),
		pattern->GetYaxis()->GetXmax()
		);
  
  return data;
  
}

TH1D *SetBins(TH1D *data, const TH1D *pattern){
  data->SetBins(
		pattern->GetNbinsX(),
		pattern->GetXaxis()->GetXmin(),
		pattern->GetXaxis()->GetXmax());
  return data;
}

bool compatible (const TH2 *h1, const TH2 *h2){
  if (h1->GetNbinsX()!=h2->GetNbinsX()) return false;
  if (h1->GetNbinsY()!=h2->GetNbinsY()) return false;

  double eps=1e-5;
  if (fabs(h1->GetXaxis()->GetXmin()-
	   h2->GetXaxis()->GetXmin())>eps) return false;
  if (fabs(h1->GetXaxis()->GetXmax()-
	   h2->GetXaxis()->GetXmax())>eps) return false;
  if (fabs(h1->GetYaxis()->GetXmin()-
	   h2->GetYaxis()->GetXmin())>eps) return false;
  if (fabs(h1->GetYaxis()->GetXmax()-
	   h2->GetYaxis()->GetXmax())>eps) return false;
  
  return true;
}

double dotProduct(const TH2D *h1, const TH2D *h2){
  TH2D hT;
  h1->Copy(hT);
  hT.Multiply(h2);
  return hT.Integral();
}

TH2 *Apply(TF1 *f1, TH2 *data){

  for (int i=0;i<data->GetNbinsX();i++){
    for (int j=0;j<data->GetNbinsY();j++){
      Stat_t fu=data->GetBinContent(data->GetBin(i+1,j+1));
      fu=f1->Eval(fu);
      data->SetBinContent(data->GetBin(i+1,j+1),fu);
    }
  }
  return data;
}


TF1 *fSqrt(){
  TF1 *f1=dynamic_cast<TF1 *>(gROOT->GetListOfFunctions()->FindObject("fSqrt"));
  if (!f1) f1=new TF1("fSqrt","sqrt(x)",0,1000);
  return f1;
}

string get_setup_path(){
  ostringstream os;

#ifdef CONFIG_PATH
  os<<CONFIG_PATH;
#else
  os<<".";
#endif
  
  return os.str();
}


bool dirExists(string dir){
  return(opendir(dir.c_str()));
}

void verifyDir(string dir){
  if (dirExists(dir)) return;
  if (mkdir(dir.c_str(), 00755)){
    cout<<"Could not create dir "<<dir<<endl;
  }
}

string getRandomTag(){
  TRandom3 ran(0);

  unsigned int f=static_cast<unsigned int>(ceil(ran.Rndm()*0xffffffff));
  ostringstream os;
  os<<setw(8)<<setfill('0')<<hex<<f<<dec;
  return os.str();
}

TF1* getGaus(){
  string fGaus("fGaus");
  TF1 *f1=dynamic_cast<TF1 *>(gROOT->FindObject(fGaus.c_str()));
  if (!f1) f1=new TF1(fGaus.c_str(),"gaus",0,1);
  return f1;
}

TF2 *FitGaus2D(TH2D *h2, double s){
   //do 2d gaussian for lines inbetween
  
  //find the peak
  int imax[3];
  h2->GetMaximumBin(imax[0],imax[1],imax[2]);
  double x0=h2->GetXaxis()->GetBinCenter(imax[0]);
  double y0=h2->GetYaxis()->GetBinCenter(imax[1]);
  


  //approximate point is (avg[i],m.yavg[j])
  //double stdev=sqrt(var[i]);
  double stdev=s;
  string fGaus2Name("fGaus2");

  TF2 *fG2=dynamic_cast<TF2 *>(gROOT->FindObject(fGaus2Name.c_str()));
  if (!fG2) fG2=new TF2(fGaus2Name.c_str(),"[0]*exp(-(x-[1])*(x-[1])/2/[2]-(y-[3])*(y-[3])/2/[4])",0,1,0,1);

  fG2->SetRange(x0-1.5*s,y0-1.5*s,x0+1.5*s,y0+1.5*s);
  fG2->SetParameter(0,h2->GetBinContent(h2->GetBin(imax[0],imax[1],imax[2])));
  fG2->SetParameter(1,x0);
  fG2->SetParameter(2,s*s);
  fG2->SetParameter(3,y0);
  fG2->SetParameter(4,s*s);
  cout<<"Gaus in: ("<<x0<<','<<y0<<"); range x["<<x0-1.5*s<<','<<x0+1.5*s<<"]"
      <<" y["<<fG2->GetYmin()<<','<<fG2->GetYmax()<<"]"<<" pars:";
  for (int k=0;k<fG2->GetNpar();k++) {
    if (k) cout<<' ';
    cout<<fG2->GetParameter(k);
  }
  cout<<endl;
  int ok=h2->Fit(fG2,"RQN");
  return fG2;
}

TGraph *drawProjection(const TH2D *h2, TLine *aL, int n){
  TGraph *gr=new TGraph(n);
  return drawProjection(*gr,h2,aL,n);
}

TGraph *drawProjection(TGraph& gr, const TH2D *h2, TLine *aL, int n){


  double v[2];
  v[0]=aL->GetX2()-aL->GetX1();
  v[1]=aL->GetY2()-aL->GetY1();
  double vv=v[0]*v[0]+v[1]*v[1];

  double v0[2];
  v0[0]=v[1]*(aL->GetX2()*v[1]-aL->GetY2()*v[0])/vv;
  v0[1]=v[0]*(aL->GetY2()*v[0]-aL->GetX2()*v[1])/vv;
  
  double s0=0;
  double len=sqrt(vv);
  if (v[0]!=0) s0=sqrt(vv)*(v0[0]-aL->GetX1())/v[0];
  else s0=sqrt(vv)*(v0[1]-aL->GetY1())/v[1];
  
  gr.Set(n);

  for (int i=0;i<gr.GetN();i++){
    double s=(i+0.5)/n*len-s0;
    double x=aL->GetX1()+(i+0.5)/n*v[0];
    double y=aL->GetY1()+(i+0.5)/n*v[1];

    //interpolate value at (x,y)
    
    double val=interpolate(h2,x,y);
    gr.SetPoint(i,s,val);
  }
  return &gr;
}

double interpolate(const TH2D *h2, double x, double y){


  //interpolate value at (x,y)
  double xq=x+0.5*h2->GetXaxis()->GetBinWidth(1);
  double yq=y+0.5*h2->GetYaxis()->GetBinWidth(1);
  
  int ix=h2->GetXaxis()->FindBin(xq);
  int iy=h2->GetYaxis()->FindBin(yq);
  
  double dx=x-h2->GetXaxis()->GetBinCenter(ix-1);
  if (fabs(dx)<10*numeric_limits<double>::epsilon()) dx=0;
  double tx=dx/h2->GetXaxis()->GetBinWidth(ix);
  
  double dy=y-h2->GetYaxis()->GetBinCenter(iy-1);
  if (fabs(dy)<10*numeric_limits<double>::epsilon()) dy=0;
  double ty=dy/h2->GetYaxis()->GetBinWidth(iy);
  
  if (tx<0 || tx>1 || ty<0 || ty>1){
    cout<<"Error obtaining tx; "
	<<"x="<<x
	<<" y="<<y
	<<" ix="<<ix<<" c(ix)="<<h2->GetXaxis()->GetBinCenter(ix)
	<<" iy="<<iy<<" c(iy)="<<h2->GetYaxis()->GetBinCenter(iy)
	<<" (ix-1)="<<ix-1<<" c(ix-1)="<<h2->GetXaxis()->GetBinCenter(ix-1)
	<<" (iy-1)="<<iy<<" c(iy-1)="<<h2->GetYaxis()->GetBinCenter(iy-1)
	<<" tx="<<tx<<" ty="<<ty
	<<" dx="<<dx
	<<" dy="<<dy
	<<endl;
  }
  Stat_t w00=h2->GetBinContent(h2->GetBin(ix-1,iy-1));
  Stat_t w01=h2->GetBinContent(h2->GetBin(ix,iy-1));
  Stat_t w10=h2->GetBinContent(h2->GetBin(ix-1,iy));
  Stat_t w11=h2->GetBinContent(h2->GetBin(ix,iy));
  double val=(1-tx)*(1-ty)*w00+
    tx*(1-ty)*w01+
    (1-tx)*ty*w10+
    tx*ty*w11;
  return val;
}

TH2D *constCopy(TH2D *h2, const TH2D *h1){
  if (!compatible(h2,h1)) SetBins(h2,h1);
  h1->TArrayD::Copy(*h2);
  return h2;
}

string get_dir(string file){
  string::size_type s1=file.rfind('/');
  string dir=string(".");
  if (s1!=string::npos)
    dir=file.substr(0,s1);
  if (0) cout<<"File from directory "<<dir<<endl;
  return dir;
}

bool isInt(string word, int& val){
  char *end_ptr;
  val=strtol(word.c_str(),&end_ptr,10);
  return (*end_ptr=='\0');
}

TH2D *clear_negative(TH2D *h){
  for (int i=0;i<h->GetNbinsX();i++){
    for (int j=0;j<h->GetNbinsY();j++){
      int gbin=h->GetBin(i+1,j+1);
      double v=h->GetBinContent(gbin);
      if (v<0) h->SetBinContent(gbin,0);
    }
  }
  return h;
}

TGraph& toGraph(const TH1D& h1, TGraph& gr, bool sym){
  gr.Set(h1.GetNbinsX());
  if (sym) {
    gr.Set(2*h1.GetNbinsX());
    if (fabs(h1.GetBinCenter(1)/h1.GetBinWidth(1))<1e-3) gr.Set(2*h1.GetNbinsX()-1);
  }
  for (int i=0;i<h1.GetNbinsX();i++){
    if (!sym){
      gr.SetPoint(i,h1.GetBinCenter(i+1),h1[i+1]);
      continue;
    }
    //symetrized histogram -> unfold
    int ip=h1.GetNbinsX()-1-i;
    int k0=ip;
    int k1=gr.GetN()-1-ip;
    gr.SetPoint(k1,h1.GetBinCenter(i+1),h1[i+1]);
    if (k1==k0) continue;
    gr.SetPoint(k0,-h1.GetBinCenter(i+1),h1[i+1]);
  }
  
  return gr;
}

TGraph *makeGraph(TGraph *gr, TH2D *data, int dir){
  TAxis *ax=data->GetXaxis();
  if (dir) ax=data->GetYaxis();

  gr->Set(0);
  int ipts=0;
  for (int i=0;i<ax->GetNbins();i++){
    TH1D *hP=0;
    if (dir) hP=data->ProjectionX("px",i+1,i+1);
    else hP=data->ProjectionY("py",i+1,i+1);
    if (hP->Integral()<1) continue;
    gr->SetPoint(ipts++,ax->GetBinCenter(i+1),
		 hP->Integral()/hP->GetNbinsX());
  }
  return gr;
}


TGraph *makeGraph(TGraph *gr, TH1D *h1, double eps){
  //use nonzero entries from h1 to make points for TGraph
  double max=eps*h1->GetBinContent(h1->GetMaximumBin());
  gr->Set(1000);
  int n=0;
  for (int i=0;i<h1->GetNbinsX();i++){
    if ((*h1)[i+1]<max) continue;
    gr->SetPoint(n,h1->GetBinCenter(i+1),(*h1)[i+1]); 
    n++;
  }
  gr->Set(n);
  return gr;
  //gr->Sort();
}

TH1D *shiftHisto(TH1D& h1, int off){
  TH1D ht;
  h1.Copy(ht);
  h1.Reset();

  for (int i=0;i<h1.GetNbinsX();i++){
    int j=i+off;
    if (j<0) continue;
    if (!(j<ht.GetNbinsX())) continue;
    
    h1.AddBinContent(j+1,ht[i+1]);
  }
  
  return &h1;
}
  
string dirname(string data){
  char *dt=new char[data.length()+1];
  data.copy(dt,string::npos);
  dt[data.length()]=0;
  return string(dirname(dt));
}

// int nF(string line){
//   string d;
//   istringstream is(line);

//   int n=0;
//   for (;is;n++)
//     is>>d;

//   return n-1;
// }

void dumpGraph(TGraph& gr, string file){
  ofstream of(file.c_str());
  for (int i=0;i<gr.GetN();i++){
    of<<gr.GetX()[i]<<' '<<gr.GetY()[i]<<endl;
  }
  of.close();
}

void dumpGraph(TGraphErrors& gr, string file){
  ofstream of(file.c_str());
  for (int i=0;i<gr.GetN();i++){
    of<<gr.GetX()[i]<<' '<<gr.GetY()[i]<<' '<<gr.GetEX()[i]<<' '<<gr.GetEY()[i]<<endl;
  }
  of.close();
}

void readGraph(TGraph& gr, string file){
  cout<<"Reading "<<file<<endl;
  gr=TGraph(file.c_str());
}

void readGraph(TGraphErrors& gr, string file){
  gr=TGraphErrors(file.c_str());
}

double getMeanGraph(TGraph& gr){
  double s=0,sy=0;
  for (int i=0;i<gr.GetN();i++){
    s+=gr.GetX()[i]*gr.GetY()[i];
    sy+=gr.GetY()[i];
  }
  return s/sy;
}

void scaleGraph(TGraph& gr, TGraph& gr1){
  double sy=0,sy1=0;

  for (int i=0;i<gr.GetN();i++)
    sy+=gr.GetY()[i];
  sy/=gr.GetN();

  for (int i=0;i<gr1.GetN();i++)
    sy1+=gr1.GetY()[i];
  sy1/=gr1.GetN();
  
  double q=sy/sy1;
  cout<<"Scale="<<q<<endl;
  
  for (int i=0;i<gr1.GetN();i++)
    gr1.GetY()[i]*=q;
}

void invertGraph(TGraph& gr){
  for (int i=0;i<gr.GetN();i++)
    gr.GetY()[i]=1/gr.GetY()[i];
}

void ratioGraph(TGraph& gr, TGraph& gr1){
  for (int i=0;i<gr.GetN();i++){
    gr1.GetY()[i]=gr1.GetY()[i]/gr.GetY()[i];
  }

}

void invertRatioGraph(TGraph& gr, TGraph& gr1){
  for (int i=0;gr.GetN();i++){
    gr1.GetY()[i]*=gr.GetY()[i];
  }
}

void Add(TH2D& h2, TH2D& hA){
  for (int i=0;i<h2.GetNbinsX();i++){
    for (int j=0;j<h2.GetNbinsY();j++){
      double x=h2.GetXaxis()->GetBinCenter(i+1);
      double y=h2.GetYaxis()->GetBinCenter(j+1);
      
      h2.AddBinContent
	(h2.GetBin(i+1,j+1),
	 hA.Interpolate(x,y));
    }
  }
}
		       
double rIntegral(const TH2D& h2, double R, int n){
  static TRandom3 ran;
  double s=0;
  for (int i=0;i<n;i++){
    double r=sqrt(ran.Rndm())*R;
    double phi=2*TMath::Pi()*ran.Rndm();
    double x=r*cos(phi);
    double y=r*sin(phi);
    int gbin=h2.FindFixBin(x,y);
    s+=h2.GetBinContent(gbin);
  }
  s/=n;
  s*=TMath::Pi()*R*R;
  s/=h2.GetXaxis()->GetBinWidth(1)*h2.GetYaxis()->GetBinWidth(1);
  return s;
}
