#ifndef __UTILS__
#define __UTILS__

#include "TROOT.h"
#include "TH2D.h"
#include "TF1.h"
#include <string>
#include <iomanip>
#include "TF2.h"
#include "TGraph.h"
#include "TLine.h"
#include "TGraphErrors.h"

template<class T> 
T* getHistogram(const char *name){
  T* h1=dynamic_cast<T*>(gROOT->GetList()->FindObject(name));
  if (!h1) h1=new T();
  h1->SetName(name);
  h1->SetTitle(name);
  h1->SetDirectory(gROOT);
  return h1;
}

TH2D *SetBins(TH2D *data, const TH2D *pattern);
bool compatible (const TH2 *h1, const TH2 *h2);

double dotProduct(const TH2D *h1, const TH2D *h2);
//a dot product of 2 sinograms viewed as vectors

TF1 *fSqrt();
TH2 *Apply(TF1 *f1, TH2 *data);
//this=f1(this) for each bin in the sinogram

std::string get_setup_path();
bool dirExists(std::string dir);
void verifyDir(std::string dir);

std::string getRandomTag();

TF2* FitGaus2D(TH2D *h2, double s);
TF1* getGaus();

//works for T with T::operator=(const T& obj)  and T::T() like TObject

//use copy to copy well-identified objects
template <class T> 
void copy(T **into, const T *orig){
  if (!orig){
    delete (*into);
    (*into)=0;
    return;
  }
  if (!(*into)) (*into)=new T();
  *(*into)=(*orig);
}

//use clone to copy polymorphic class; clone insures the proper object is duplicated
template <class T> 
void clone(T **into, const T *orig){
  delete (*into);
  (*into)=0;
  if (!orig){
    return;
  }
  (*into)=orig->Clone();
}

//draw a projection of histogram h2 along the line aL using
//interpolation at n points along the line; shift 0 coordinate to
//point of closest approach on the line
TGraph *drawProjection(const TH2D *h2, TLine *aL, int n=100);
TGraph *drawProjection(TGraph& gr, const TH2D *h2, TLine *aL, int n=100);

double interpolate(const TH2D *h2, double xq, double yq);

TH1D *SetBins(TH1D *data, const TH1D *pattern);

TH2D *constCopy(TH2D *h2, const TH2D *h1);//copy data from h2 to h1 avoiding TH2D::Copy routine

//extract directory part of a file name
std::string get_dir(std::string file);

bool isInt(std::string t, int& v);

TH2D *clear_negative(TH2D *h);

TGraph& toGraph(const TH1D& h1, TGraph& gr, bool sym=0);

TGraph *makeGraph(TGraph *gr, TH1D *h1, double eps=1e-8);
TGraph *makeGraph(TGraph *gr, TH2D *data, int dir=1);

TH1D *shiftHisto(TH1D &h1, int off);

std::string dirname(std::string file);

void dumpGraph(TGraph& gr, std::string file);
void dumpGraph(TGraphErrors& gr, std::string file);
void readGraph(TGraph& gr, std::string file);
void readGraph(TGraphErrors& gr, std::string file);

double getMeanGraph(TGraph& gr);
void scaleGraph(TGraph& gr, TGraph& gr1);
void invertGraph(TGraph& gr);

void ratioGraph(TGraph& gr, TGraph& gr1);
void invertRatioGraph(TGraph& gr, TGraph& gr1);

//add using interpolation
void Add(TH2D& h2, TH2D& hA);

double rIntegral(const TH2D& h2, double R, int n=10000);

#endif
